@@ -1,36 +1,74 @@
-/*****************************************************************************
- *****************************************************************************
- Copyright (c) 1999-2001, Intel Corporation 
+/**********************************************************************
+This software program is available to you under a choice of one of two 
+licenses. You may choose to be licensed under either the GNU General Public 
+License (GPL) Version 2, June 1991, available at 
+http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
+text of which follows:
 
- All rights reserved.
+Recipient has requested a license and Intel Corporation ("Intel") is willing
+to grant a license for the software entitled Linux Base Driver for the 
+Intel(R) PRO/100 Family of Adapters (e100) (the "Software") being provided 
+by Intel Corporation. The following definitions apply to this license:
 
- Redistribution and use in source and binary forms, with or without 
- modification, are permitted provided that the following conditions are met:
+"Licensed Patents" means patent claims licensable by Intel Corporation which 
+are necessarily infringed by the use of sale of the Software alone or when 
+combined with the operating system referred to below.
 
- 1. Redistributions of source code must retain the above copyright notice, 
- this list of conditions and the following disclaimer.
+"Recipient" means the party to whom Intel delivers this Software.
 
- 2. Redistributions in binary form must reproduce the above copyright notice,
- this list of conditions and the following disclaimer in the documentation 
- and/or other materials provided with the distribution.
+"Licensee" means Recipient and those third parties that receive a license to 
+any operating system available under the GNU Public License version 2.0 or 
+later.
 
- 3. Neither the name of Intel Corporation nor the names of its contributors 
- may be used to endorse or promote products derived from this software 
- without specific prior written permission.
+Copyright (c) 1999 - 2002 Intel Corporation.
+All rights reserved.
 
- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
- EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+The license is provided to Recipient and Recipient's Licensees under the 
+following terms.
 
- *****************************************************************************
-****************************************************************************/
+Redistribution and use in source and binary forms of the Software, with or 
+without modification, are permitted provided that the following conditions 
+are met:
+
+Redistributions of source code of the Software may retain the above 
+copyright notice, this list of conditions and the following disclaimer.
+
+Redistributions in binary form of the Software may reproduce the above 
+copyright notice, this list of conditions and the following disclaimer in 
+the documentation and/or materials provided with the distribution.
+
+Neither the name of Intel Corporation nor the names of its contributors 
+shall be used to endorse or promote products derived from this Software 
+without specific prior written permission.
+
+Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
+royalty-free patent license under Licensed Patents to make, use, sell, offer 
+to sell, import and otherwise transfer the Software, if any, in source code 
+and object code form. This license shall include changes to the Software 
+that are error corrections or other minor changes to the Software that do 
+not add functionality or features when the Software is incorporated in any 
+version of an operating system that has been distributed under the GNU 
+General Public License 2.0 or later. This patent license shall apply to the 
+combination of the Software and any operating system licensed under the GNU 
+Public License version 2.0 or later if, at the time Intel provides the 
+Software to Recipient, such addition of the Software to the then publicly 
+available versions of such operating systems available under the GNU Public 
+License version 2.0 or later (whether in gold, beta or alpha form) causes 
+such combination to be covered by the Licensed Patents. The patent license 
+shall not apply to any other combinations which include the Software. NO 
+hardware per se is licensed hereunder.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
+AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
+(INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
+ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
+AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
+TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
+OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+**********************************************************************/
 
 /**********************************************************************
 *                                                                     *
@@ -49,8 +87,6 @@
 *                                                                     *
 **********************************************************************/
 
-
-
 #include "ans_driver.h"
 #include "ans_os.h"
 #include <asm/unaligned.h>
@@ -87,7 +123,7 @@ bd_ans_os_Ioctl(device_t *dev, struct ifreq *ifr, int cmd)
     iANSsupport_t *iANSdata;
     BD_ANS_STATUS status;
     
-    /* get a pointer to the ANS data struct from the bps */
+    /* get a pointer to the ANS data struct from the ifr */
     iANSdata = ANS_PRIVATE_DATA_FIELD(bps);
 
     //DEBUGLOG("bd_ans_os_Ioctl: enter\n");
@@ -126,7 +162,7 @@ bd_ans_os_Transmit( BOARD_PRIVATE_STRUCT *bps,
     HW_TX_DESCRIPTOR *txd,
     struct sk_buff **skb )
 {
-  UINT16 vlan_id;     /* don't know if I really need this */
+    UINT16 vlan_id;     /* don't know if I really need this */
     IANS_ATTR_HEADER *attr_head = iANSGetTransmitAttributeHeader(*skb);
     iANSsupport_t *iANSdata = ANS_PRIVATE_DATA_FIELD(bps);
     
@@ -134,6 +170,14 @@ bd_ans_os_Transmit( BOARD_PRIVATE_STRUCT *bps,
     
     /* call the bd_ans_Transmit routine to setup our frame for transmit */
     if (iANSdata->attributed_mode) {
+
+        /* Check to identify misrouted frames */
+        /* An explicit check would not have been backward compatible */
+        if ((char *)(attr_head->pFirstTLV) != ( (char *)attr_head + sizeof(IANS_ATTR_HEADER) )) {
+            //printk("%s warning frame does not contain TLV data",(*skb)->dev->name);
+            return 1; // This is not an iANS attributed packet!
+        }
+
         if (bd_ans_Transmit(bps,
             iANSdata,
             attr_head->pFirstTLV,
@@ -222,10 +266,10 @@ bd_ans_os_Receive(BOARD_PRIVATE_STRUCT *bps,
 
 /* bd_ans_os_Watchdog()
 **
-**  This function will check the status fields of the ANS
+**  This function will check on the status fields of the ANS
 **  support structure and see if the status has changed since
-**  the last time that it has been checked. If it has changed,
-**  then we need to alert the ANS protocol somehow.
+**  the last time that it was checked.  If it has changed,
+**  then we need to alert the ANS protocol somehow.(TBD)
 **
 **  Arguments:  struct device *dev - pointer to the device structure
 **              BOARD_PRIVATE_STRUCT *bps - the driver's private data struct
@@ -263,6 +307,7 @@ bd_ans_os_Watchdog(device_t *dev, BOARD_PRIVATE_STRUCT *bps)
     }
     
     /* if we are here, we need to send a status change notification */
+    
     /* as far as I can tell, since it hasn't been defined yet,
      * the thing to do here is to call netdev_state_change(dev).
      * this is a synchronous call to a registered chain of who knows
@@ -463,3 +508,76 @@ bd_ans_os_ReserveSpaceForANS(struct sk_buff *skb)
     skb_reserve(skb, BD_ANS_INFO_SIZE);
 }
        
+BD_ANS_STATUS bd_ans_os_ProcessRequest(BOARD_PRIVATE_STRUCT *bps, 
+                                       iANSsupport_t *iANSdata,
+                                       IANS_BD_PARAM_HEADER *header)
+{
+    switch (header->Opcode) {
+    case IANS_OP_ANS_SET_CB:
+        DEBUGLOG("bd_ans_ProcessRequest: ans set callbacks\n");
+        return (bd_ans_os_SetCallback(bps, header));
+    default:
+        DEBUGLOG1("bd_ans_os_ProcessRequest: unknown op code = %d\n", header->Opcode);
+        return (BD_ANS_FAILURE);
+    }
+}
+
+BD_ANS_STATUS bd_ans_os_ExtendedGetCapability(BOARD_PRIVATE_STRUCT *bps,
+                                              iANSsupport_t *iANSdata,
+                                              IANS_BD_PARAM_HEADER *header)
+{
+    ((IANS_BD_PARAM_EXT_CAP *)header)->BDAllAvailableRouting = 
+        IANS_ROUTING_NOT_SUPPORTED;      
+    /* get routing capabilities */
+    if (BD_ANS_OS_CAN_ROUTE_RX(bps) == BD_ANS_TRUE) {
+        ((IANS_BD_PARAM_EXT_CAP *)header)->BDAllAvailableRouting |=
+            IANS_ROUTING_RX_PROTOCOL;
+    } 
+    ((IANS_BD_PARAM_EXT_CAP *)header)->BDFlags = iANSdata->flags.bd_flags;
+
+    return BD_ANS_SUCCESS;        
+}
+
+BD_ANS_STATUS bd_ans_os_ExtendedSetMode(BOARD_PRIVATE_STRUCT *bps,
+                                        iANSsupport_t *iANSdata,
+                                        IANS_BD_PARAM_HEADER *header)
+{
+    IANS_BD_PARAM_EXT_SET_MODE *request = (IANS_BD_PARAM_EXT_SET_MODE *)header;
+    if (request->BDIansRoutingMode & IANS_ROUTING_RX_PROTOCOL)
+        iANSdata->routing_mode = IANS_ROUTING_ON;
+    else
+        iANSdata->routing_mode = IANS_ROUTING_OFF;
+        
+   return BD_ANS_SUCCESS;
+}
+
+BD_ANS_STATUS bd_ans_os_ActivateFastPolling(BOARD_PRIVATE_STRUCT *bps,
+                                            iANSsupport_t *iANSdata)
+{
+    /* initialize the previous status with the current status so
+     * that we don't send any bogus status change indications
+     */
+    bd_ans_drv_UpdateStatus(bps);
+    bd_ans_FillStatus(bps, iANSdata, &(iANSdata->prev_status));
+
+    /* tell the driver that we need to start checking the
+     * status
+     */    
+    iANSdata->timer_id = bd_ans_drv_StartWatchdog(bps);
+    
+    /* a non-zero timer_id indicates that the driver has
+     * started the Watchdog.
+     */
+    if (iANSdata->timer_id == 0)
+        return BD_ANS_FAILURE;
+        
+    return BD_ANS_SUCCESS; 
+}
+
+
+BD_ANS_STATUS bd_ans_os_GetAllCapabilities(BOARD_PRIVATE_STRUCT *bps,
+                                            iANSsupport_t *iANSdata)
+{
+    iANSdata->flags.bd_flags = BD_ANS_HW_FLAGS(bps);
+    return BD_ANS_SUCCESS;
+}
