@@ -1,36 +1,74 @@
-/*****************************************************************************
- *****************************************************************************
- Copyright (c) 1999-2001, Intel Corporation 
+/**********************************************************************
+This software program is available to you under a choice of one of two 
+licenses. You may choose to be licensed under either the GNU General Public 
+License (GPL) Version 2, June 1991, available at 
+http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
+text of which follows:
 
- All rights reserved.
+Recipient has requested a license and Intel Corporation ("Intel") is willing
+to grant a license for the software entitled Linux Base Driver for the 
+Intel(R) PRO/100 Family of Adapters (e100) (the "Software") being provided 
+by Intel Corporation. The following definitions apply to this license:
 
- Redistribution and use in source and binary forms, with or without 
- modification, are permitted provided that the following conditions are met:
+"Licensed Patents" means patent claims licensable by Intel Corporation which 
+are necessarily infringed by the use of sale of the Software alone or when 
+combined with the operating system referred to below.
 
- 1. Redistributions of source code must retain the above copyright notice, 
- this list of conditions and the following disclaimer.
+"Recipient" means the party to whom Intel delivers this Software.
 
- 2. Redistributions in binary form must reproduce the above copyright notice,
- this list of conditions and the following disclaimer in the documentation 
- and/or other materials provided with the distribution.
+"Licensee" means Recipient and those third parties that receive a license to 
+any operating system available under the GNU Public License version 2.0 or 
+later.
 
- 3. Neither the name of Intel Corporation nor the names of its contributors 
- may be used to endorse or promote products derived from this software 
- without specific prior written permission.
+Copyright (c) 1999 - 2002 Intel Corporation.
+All rights reserved.
 
- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
- EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+The license is provided to Recipient and Recipient's Licensees under the 
+following terms.
 
- *****************************************************************************
-****************************************************************************/
+Redistribution and use in source and binary forms of the Software, with or 
+without modification, are permitted provided that the following conditions 
+are met:
+
+Redistributions of source code of the Software may retain the above 
+copyright notice, this list of conditions and the following disclaimer.
+
+Redistributions in binary form of the Software may reproduce the above 
+copyright notice, this list of conditions and the following disclaimer in 
+the documentation and/or materials provided with the distribution.
+
+Neither the name of Intel Corporation nor the names of its contributors 
+shall be used to endorse or promote products derived from this Software 
+without specific prior written permission.
+
+Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
+royalty-free patent license under Licensed Patents to make, use, sell, offer 
+to sell, import and otherwise transfer the Software, if any, in source code 
+and object code form. This license shall include changes to the Software 
+that are error corrections or other minor changes to the Software that do 
+not add functionality or features when the Software is incorporated in any 
+version of an operating system that has been distributed under the GNU 
+General Public License 2.0 or later. This patent license shall apply to the 
+combination of the Software and any operating system licensed under the GNU 
+Public License version 2.0 or later if, at the time Intel provides the 
+Software to Recipient, such addition of the Software to the then publicly 
+available versions of such operating systems available under the GNU Public 
+License version 2.0 or later (whether in gold, beta or alpha form) causes 
+such combination to be covered by the Licensed Patents. The patent license 
+shall not apply to any other combinations which include the Software. NO 
+hardware per se is licensed hereunder.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
+AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
+(INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
+ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
+AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
+TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
+OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+**********************************************************************/
 
 /**********************************************************************
 *                                                                     *
@@ -52,25 +90,25 @@
 #include "e100_config.h"
 
 /* global variables */
-static uint8_t e100_cfg_parm6 = CFG_BYTE_PARM6;
+static u8 e100_cfg_parm6 = CFG_BYTE_PARM6;
 
-static uint8_t e100_rx_fifo_lmt =
+static u8 e100_rx_fifo_lmt =
 (RX_FIFO_LMT < 0) ? DEFAULT_RX_FIFO_LIMIT : ((RX_FIFO_LMT >
                                               15) ? DEFAULT_RX_FIFO_LIMIT : RX_FIFO_LMT);
 
-static uint8_t e100_tx_fifo_lmt =
+static u8 e100_tx_fifo_lmt =
 (TX_FIFO_LMT < 0) ? DEFAULT_TX_FIFO_LIMIT : ((TX_FIFO_LMT >
                                               7) ? DEFAULT_TX_FIFO_LIMIT : TX_FIFO_LMT);
 
-static uint8_t e100_rx_dma_cnt =
+static u8 e100_rx_dma_cnt =
 (RX_DMA_CNT < 0) ? CB_557_CFIG_DEFAULT_PARM4 : ((RX_DMA_CNT >
                                                  63) ? CB_557_CFIG_DEFAULT_PARM4 : RX_DMA_CNT);
 
-static uint8_t e100_tx_dma_cnt =
+static u8 e100_tx_dma_cnt =
 (TX_DMA_CNT < 0) ? CB_557_CFIG_DEFAULT_PARM5 : ((TX_DMA_CNT >
                                                  63) ? CB_557_CFIG_DEFAULT_PARM5 : TX_DMA_CNT);
 
-static uint8_t e100_urun_retry =
+static u8 e100_urun_retry =
 (TX_UR_RETRY < 0) ? CB_557_CFIG_DEFAULT_PARM5 : ((TX_UR_RETRY >
                                                   3) ? CB_557_CFIG_DEFAULT_PARM5 : TX_UR_RETRY);
 
@@ -101,7 +139,6 @@ static int e100_read_align_enable = 0;
 
 
 /* extern variables */
-extern int e100_enhanced_tx_enable;
 extern int e100_speed_duplex[MAX_NIC];
 extern int flow_control[MAX_NIC];
 extern int XsumRX[MAX_NIC];
@@ -118,8 +155,6 @@ extern int XsumRX[MAX_NIC];
  */
 void e100_config_init(bd_config_t *bdp)
 {
-    bdd_t          *bddp = bdp->bddp;   /* stores all adapter specific info */
-    
     /* initialize config block */
     memset(bdp->config, 0x00, sizeof(bdp->config));
 
@@ -144,12 +179,12 @@ void e100_config_init(bd_config_t *bdp)
 
     /* Change for 82558 enhancement */
     /* Set the Tx and Rx Fifo limits */
-    if ((bddp->flags & IS_BACHELOR) && (e100_tx_fifo_lmt < 8)) {
+    if ((bdp->flags & IS_BACHELOR) && (e100_tx_fifo_lmt < 8)) {
         e100_tx_fifo_lmt = 8;   /* set 8 as the minimum */
     }
     
     bdp->config[1] = 
-        (uint8_t) (BIT_7 | (e100_tx_fifo_lmt << 4) | e100_rx_fifo_lmt);
+        (u8) (BIT_7 | (e100_tx_fifo_lmt << 4) | e100_rx_fifo_lmt);
     
     e100_config_ifs(bdp);
     
@@ -157,14 +192,14 @@ void e100_config_init(bd_config_t *bdp)
     /* MWI enable. This should be turned on only if enabled in * e100.c and
      * if the adapter is a 82558/9 and if the PCI command reg. * has enabled
      * the MWI bit. */
-    if ((bddp->flags & IS_BACHELOR) && (e100_MWI_enable)) {
+    if ((bdp->flags & IS_BACHELOR) && (e100_MWI_enable)) {
         bdp->config[3] |= CB_CFIG_MWI_EN;
     }
     
     /* Read Align/Write Terminate on cache line. This should be * turned on
      * only if enabled in e100.c and if the adapter is a 82558/9 * and if the 
      * system is cache line oriented. */
-    if ((bddp->flags & IS_BACHELOR) && (e100_read_align_enable))
+    if ((bdp->flags & IS_BACHELOR) && (e100_read_align_enable))
         bdp->config[3] |= CB_CFIG_READAL_EN | CB_CFIG_TERMCL_EN;
     
     /* Set the Tx and Rx DMA maximum byte count fields. */
@@ -183,13 +218,13 @@ void e100_config_init(bd_config_t *bdp)
         bdp->config[6] = CB_557_CFIG_DEFAULT_PARM6;
     }
     
-    if ((bddp->flags & IS_BACHELOR) && (e100_enhanced_tx_enable)) {
+    if (bdp->flags & IS_BACHELOR) {
         bdp->config[6] &= ~CB_CFIG_EXT_TCB_DIS;
     }
     
     /*
      * Enable extended statistical counters (82558 and up) and TCO counters
-     * (82559 and up) and set the statistical counters' mode in bddp 
+     * (82559 and up) and set the statistical counters' mode in bdp 
      *  
      *  stat. mode      |    TCO stat. bit (2)  |  Extended stat. bit (5)
      * ------------------------------------------------------------------
@@ -202,23 +237,23 @@ void e100_config_init(bd_config_t *bdp)
      *  Reserved        |       1               |         0
      * ------------------------------------------------------------------
      */
-    if (bddp->flags & IS_BACHELOR) {  /* This is 558 and up - enable extended counters */
+    if (bdp->flags & IS_BACHELOR) {  /* This is 558 and up - enable extended counters */
         
         if (bdp->rev_id >= D101MA_REV_ID) {   /* this is 559 and up - enable TCO counters */
             bdp->config[6] |= CB_CFIG_TCO_STAT;
             bdp->config[6] |= CB_CFIG_EXT_STAT_DIS;
-            bddp->stat_mode = E100_TCO_STATS;
+            bdp->stat_mode = E100_TCO_STATS;
         
         } else {  /* this is 558 */
             bdp->config[6] &= ~CB_CFIG_TCO_STAT;
             bdp->config[6] &= ~CB_CFIG_EXT_STAT_DIS;
-            bddp->stat_mode = E100_EXTENDED_STATS;
+            bdp->stat_mode = E100_EXTENDED_STATS;
         }
         
     } else {  /* this is 557 */
         bdp->config[6] &= ~CB_CFIG_TCO_STAT;
        	bdp->config[6] |= CB_CFIG_EXT_STAT_DIS;
-        bddp->stat_mode = E100_BASIC_STATS;
+        bdp->stat_mode = E100_BASIC_STATS;
     }
     
     /* Set up number of retries after under run */
@@ -228,7 +263,7 @@ void e100_config_init(bd_config_t *bdp)
     /* Change for 82558 enhancement */
     /* Dynamic TBD. Should be turned on only if enabled in e100.c and if
      * the adapter is a 82558/9. */
-    if ((bddp->flags & IS_BACHELOR) && (e100_enhanced_tx_enable))
+    if (bdp->flags & IS_BACHELOR)
         bdp->config[7] |= CB_CFIG_DYNTBD_EN;
     
     /* Setup for MII or 503 operation.  The CRS+CDT bit should only be set */
@@ -243,7 +278,7 @@ void e100_config_init(bd_config_t *bdp)
     
     /* Change for 82558 enhancement */
     /* enable flow control only if not 557 */
-    if ((bddp->flags & IS_BACHELOR) && (flow_control[bdp->bd_number] == TRUE)) {
+    if ((bdp->flags & IS_BACHELOR) && (flow_control[bdp->bd_number] == TRUE)) {
         bdp->config[16] = DFLT_FC_DELAY_LSB;
         bdp->config[17] = DFLT_FC_DELAY_MSB;
         /* Removed CB_CFIG_TX_FC_EN frm line below. This bit has to be 0 to 
@@ -263,12 +298,8 @@ void e100_config_init(bd_config_t *bdp)
     /* Enable checksum offloading if we are on a supported adapter. */
     if ((bdp->rev_id >= D101MA_REV_ID) && (bdp->rev_id < D102_REV_ID) &&
         (XsumRX[bdp->bd_number] == TRUE) && (bdp->ppci_dev->device != 0x1209)) {
-        
-#ifdef __i386__
-        bdp->checksum_offload_enabled = 1;
+        bdp->flags |= DF_CSUM_OFFLOAD;
         bdp->config[9] |= 1;
-#endif /* __i386__ */
-
     } else if (bdp->rev_id >= D102_REV_ID) {
         /* The D102 chip allows for 32 config bytes.  This value is
            supposed to be in Byte 0.  Just add the extra bytes to
@@ -286,7 +317,7 @@ void e100_config_init(bd_config_t *bdp)
         
         /* set the flag if checksum offloading was enabled */
         if (XsumRX[bdp->bd_number] == TRUE) {
-            bdp->checksum_offload_enabled = 1;
+            bdp->flags |= DF_CSUM_OFFLOAD;
         }
     }
 }
@@ -306,12 +337,10 @@ void e100_config_init(bd_config_t *bdp)
  *      B_TRUE - If the configure command was successfully issued and completed
  *      B_FALSE - If the configure command failed to complete properly
  */
-boolean_t
+unsigned char
 e100_force_config(bd_config_t *bdp)
 {
-    unsigned long   lock_flag_conf;
-    
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     bdp->config[0] = CB_CFIG_BYTE_COUNT;
     if (bdp->rev_id >= D102_REV_ID) {
@@ -321,7 +350,7 @@ e100_force_config(bd_config_t *bdp)
         bdp->config[0] += CB_CFIG_D102_BYTE_COUNT;
     }
 
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
     // although we call config outside the lock, there is no
     // race condition because config byte count has maximum value
     return e100_config(bdp);
@@ -342,78 +371,42 @@ e100_force_config(bd_config_t *bdp)
  *      B_TRUE - If the configure command was successfully issued and completed
  *      B_FALSE - If the configure command failed to complete properly
  */
-boolean_t
+unsigned char
 e100_config(bd_config_t *bdp)
 {
-        bdd_t          *bddp = bdp->bddp;   /* stores all adapter specific info */
         pcb_header_t    pntcb_hdr;
-        int             timeout;
-        unsigned long   lock_flag_bd;
-        unsigned long   lock_flag_conf;
-        boolean_t	res;
+        unsigned char	res;
 
         spin_lock_bh(&bdp->bd_tx_lock);
 
-        timeout = 1000*500;
+        pntcb_hdr = (pcb_header_t) bdp->pntcb;   /* get hdr of non tcb cmd */
 
-        if (bdp->bddp->last_tcbp) {
-                E100_WAIT_FOR_CONDITION(timeout,
-                                        (bdp->bddp->last_tcbp->tcb_hdr.cb_status & 
-                                         __constant_cpu_to_le16(CB_STATUS_COMPLETE)));
-        }        
-        E100_WAIT_FOR_CONDITION(timeout,
-                                ((readw(&bddp->scbp->scb_status) & SCB_CUS_MASK)
-                                 != SCB_CUS_ACTIVE));
-        if(timeout <= 0) {
-                spin_unlock_bh(&bdp->bd_tx_lock);
-                e100_tx_notify_start(bdp);
-                printk(KERN_WARNING "%s: Config failed\n",bdp->device->name);
-                return B_FALSE;  
-        }
-        spin_lock_irqsave(&bdp->bd_lock, lock_flag_bd);
-        spin_lock_irqsave(&bdp->config_lock, lock_flag_conf);
-        
-        pntcb_hdr = (pcb_header_t) bddp->pntcb;   /* get hdr of non tcb cmd */
-
-        /* Setup the non-transmit command block header for the configure command. */
-        pntcb_hdr->cb_status = 0;
         pntcb_hdr->cb_cmd = __constant_cpu_to_le16(CB_CONFIGURE);
-        /* Note: cb_lnk_ptr is a physical address stored in an unsigned long */
-        pntcb_hdr->cb_lnk_ptr = 0;
 
-        /* Copy the device's config block to the device's memory */
-        memcpy(bddp->pntcb->ntcb.config.cfg_byte, bdp->config, bdp->config[0]);
-        barrier();
-    
-        /* Wait for the SCB command word to clear before we set the * general
-         * pointer */
-        if (e100_wait_scb(bdp) != B_TRUE) {
-                spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
-                spin_unlock_irqrestore(&(bdp->bd_lock), lock_flag_bd);
+        spin_lock_bh(&bdp->config_lock);
+        
+        if (bdp->config[0] == 0) {
+                spin_unlock_bh(&bdp->config_lock);
                 spin_unlock_bh(&bdp->bd_tx_lock);
-                e100_tx_notify_start(bdp);
-                return (B_FALSE);
+                e100_tx_notify_start(bdp, SHORT_STOP);
+                return B_TRUE;
         }
 
-        bddp->next_cu_cmd = START_WAIT;
+        if (bdp->config[0] < CB_CFIG_MIN_PARAMS) {
+            bdp->config[0] = CB_CFIG_MIN_PARAMS;
+        }
+        
+        /* Copy the device's config block to the device's memory */
+        memcpy(bdp->pntcb->ntcb.config.cfg_byte, bdp->config, bdp->config[0]);
 
-        /* write the config buffer in the scb gen pointer */
-        writel(bddp->nontx_paddr, &(bddp->scbp->scb_gen_ptr));
-
-        /* Submit the configure command to the chip, and wait for it to complete. */
-        e100_exec_non_cu_cmd(bdp);
-        spin_unlock_irqrestore(&bdp->config_lock, lock_flag_conf);
-        spin_unlock_irqrestore(&bdp->bd_lock, lock_flag_bd);
-
-        if (!e100_poll_cmd(bdp)) {
-                res= B_FALSE;
-        } else {
-                bdp->config[0] = CB_CFIG_MIN_PARAMS; 
+        if ( (res = e100_exec_non_cu_cmd(bdp)) == B_TRUE) {
                 /* reset number of bytes to config next time */
-                res=B_TRUE;
+                bdp->config[0] = 0;
         }
+
+        spin_unlock_bh(&bdp->config_lock);
         spin_unlock_bh(&bdp->bd_tx_lock);
-        e100_tx_notify_start(bdp);        
+        e100_tx_notify_start(bdp, SHORT_STOP);
         return res;
 }
 
@@ -431,11 +424,9 @@ e100_config(bd_config_t *bdp)
  * Returns:
  */
 void
-e100_config_promisc(bd_config_t *bdp, boolean_t enable)
+e100_config_promisc(bd_config_t *bdp, unsigned char enable)
 {
-    unsigned long   lock_flag_conf;
-    
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     /* if in promiscuous mode, save bad frames */
     if (enable == B_TRUE) {
@@ -445,8 +436,8 @@ e100_config_promisc(bd_config_t *bdp, boolean_t enable)
             E100_CONFIG(bdp, 6);
         }
         
-        if (bdp->config[7] & (uint8_t)BIT_0) {
-            bdp->config[7] &= (uint8_t) (~BIT_0);
+        if (bdp->config[7] & (u8)BIT_0) {
+            bdp->config[7] &= (u8) (~BIT_0);
             E100_CONFIG(bdp, 7);
         }
         
@@ -462,8 +453,8 @@ e100_config_promisc(bd_config_t *bdp, boolean_t enable)
             E100_CONFIG(bdp, 6);
         }
         
-        if (!(bdp->config[7] & (uint8_t)BIT_0)) {
-            bdp->config[7] |= (uint8_t) (BIT_0);
+        if (!(bdp->config[7] & (u8)BIT_0)) {
+            bdp->config[7] |= (u8) (BIT_0);
             E100_CONFIG(bdp, 7);
         }
         
@@ -473,7 +464,7 @@ e100_config_promisc(bd_config_t *bdp, boolean_t enable)
         }
     }
     
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
 }
 
 
@@ -491,11 +482,9 @@ e100_config_promisc(bd_config_t *bdp, boolean_t enable)
  * Returns:
  */
 void
-e100_config_mulcast_enbl(bd_config_t *bdp, boolean_t enable)
+e100_config_mulcast_enbl(bd_config_t *bdp, unsigned char enable)
 {
-    unsigned long   lock_flag_conf;
-    
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     /* this flag is used to enable receiving all multicast packet */
     if (enable == B_TRUE) {
@@ -511,7 +500,7 @@ e100_config_mulcast_enbl(bd_config_t *bdp, boolean_t enable)
         }
     }
     
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
 }
 
 /*
@@ -529,15 +518,13 @@ e100_config_mulcast_enbl(bd_config_t *bdp, boolean_t enable)
 void
 e100_config_ifs(bd_config_t *bdp)
 {
-    bdd_t          *bddp = bdp->bddp;   /* stores all adapter specific info */
-    unsigned long   lock_flag_conf;
-    uint8_t         value = 0;
+    u8         value = 0;
     
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     /* IFS value is only needed to be specified at half-duplex mode */
-    if (bddp->cur_dplx_mode == HALF_DUPLEX) {
-        value = (uint8_t) bdp->ifs_value;
+    if (bdp->cur_dplx_mode == HALF_DUPLEX) {
+        value = (u8) bdp->ifs_value;
     }
     
     if (bdp->config[2] != value) {
@@ -545,7 +532,7 @@ e100_config_ifs(bd_config_t *bdp)
         E100_CONFIG(bdp, 2);
     }
     
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
 }
 
 /*
@@ -563,10 +550,8 @@ e100_config_ifs(bd_config_t *bdp)
 void
 e100_config_force_dplx(bd_config_t *bdp)
 {
-    bdd_t          *bddp = bdp->bddp;   /* stores all adapter specific info */
-    unsigned long   lock_flag_conf;
     
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     /* We must force full duplex on if we are using PHY 0, and we are */
     /* supposed to run in FDX mode.  We do this because the e100 has only */
@@ -577,11 +562,11 @@ e100_config_force_dplx(bd_config_t *bdp)
      * This is how the condition was, initially. * This has been changed so
      * that the MAC gets forced to full duplex * simply if the user has
      * forced full duplex. * * if (( bdp->phy_addr == 0 ) && (
-     * bddp->cur_dplx_mode == 2 )) */
+     * bdp->cur_dplx_mode == 2 )) */
     /* The rest of the fix is in the PhyDetect code.          */
     if ((e100_speed_duplex[bdp->bd_number] == 2) ||
         (e100_speed_duplex[bdp->bd_number] == 4) ||
-        ((bdp->phy_addr == 32) && (bddp->cur_dplx_mode == FULL_DUPLEX))) {
+        ((bdp->phy_addr == 32) && (bdp->cur_dplx_mode == FULL_DUPLEX))) {
         if (!(bdp->config[19] & CB_CFIG_FORCE_FDX)) {
             bdp->config[19] |= CB_CFIG_FORCE_FDX;
             E100_CONFIG(bdp, 19);
@@ -594,7 +579,7 @@ e100_config_force_dplx(bd_config_t *bdp)
         }
     }
     
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
 }
 
 #ifdef IANS_BASE_VLAN_TAGGING
@@ -611,7 +596,7 @@ e100_config_force_dplx(bd_config_t *bdp)
  * Returns:
  */
 static void
-e100_config_long_rx(bd_config_t *bdp, boolean_t enable)
+e100_config_long_rx(bd_config_t *bdp, unsigned char enable)
 {
     if (enable == B_TRUE) {
         if (!(bdp->config[18] & CB_CFIG_LONG_RX_OK)) {
@@ -639,7 +624,7 @@ e100_config_long_rx(bd_config_t *bdp, boolean_t enable)
  * Returns:
  */
 static void
-e100_config_vlan_drop(bd_config_t *bdp, boolean_t enable)
+e100_config_vlan_drop(bd_config_t *bdp, unsigned char enable)
 {
     if (enable == B_TRUE) {
         if (!(bdp->config[22] & CB_CFIG_VLAN_DROP_ENABLE)) {
@@ -667,11 +652,9 @@ e100_config_vlan_drop(bd_config_t *bdp, boolean_t enable)
  * Returns:
  */
 void
-e100_config_enable_tagging(bd_config_t *bdp, boolean_t enable)
+e100_config_enable_tagging(bd_config_t *bdp, unsigned char enable)
 {
-    unsigned long   lock_flag_conf;
-    
-    spin_lock_irqsave(&(bdp->config_lock), lock_flag_conf);
+    spin_lock_bh(&(bdp->config_lock));
     
     if (bdp->rev_id >= D101B0_REV_ID)
         e100_config_long_rx(bdp, enable);
@@ -679,7 +662,152 @@ e100_config_enable_tagging(bd_config_t *bdp, boolean_t enable)
     if (bdp->rev_id >= D102_REV_ID)
         e100_config_vlan_drop(bdp, enable);
     
-    spin_unlock_irqrestore(&(bdp->config_lock), lock_flag_conf);
+    spin_unlock_bh(&(bdp->config_lock));
 }
 #endif /* IANS_BASE_VLAN_TAGGING */
 
+#ifdef IDIAG_PRO_SUPPORT
+
+static unsigned char
+e100_config_loopback_change_block(bd_config_t *bdp,u32 mode)
+{
+        if( (bdp->config[10] & CB_CFIG_LOOPBACK_MODE)!= 
+            mode){
+
+                bdp->config[10] &= (~CB_CFIG_LOOPBACK_MODE);
+                bdp->config[10] |=mode;
+               	E100_CONFIG(bdp, 10);
+            	return B_TRUE;
+
+        }
+        return B_FALSE;
+}
+
+/*
+ * Procedure:   e100_config_loopback_mode
+ *
+ * Description: This routine will change the loopback mode
+ *              in the adapter's congif block, as indicated by the
+ *              loopback mode argument.
+ *
+ * Arguments:
+ *      bdp    - Ptr to this card's e100_bdconfig structure
+ *		mode   - loopback mode(internal/external/normal)
+ *
+ * Returns: True -if the config block was changed
+ */
+
+unsigned char
+e100_config_loopback_mode(bd_config_t *bdp,u32 mode)
+{
+    unsigned char	bc_changed = B_FALSE;
+
+    spin_lock_bh(&(bdp->config_lock));
+    
+    switch(mode){
+        
+    case(IDIAG_E100_DIAG_MAC_LB):
+        bc_changed = e100_config_loopback_change_block(bdp,CB_CFIG_LOOPBACK_INTERNAL);
+        break;
+        
+    case(IDIAG_E100_DIAG_EXT_LB):
+        bc_changed = e100_config_loopback_change_block(bdp,CB_CFIG_LOOPBACK_EXTERNAL);
+        break;
+    
+    case(IDIAG_E100_DIAG_TCVR_LB):
+        bc_changed = e100_config_loopback_change_block(bdp,CB_CFIG_LOOPBACK_NORMAL);
+        break;
+        
+    default:
+        printk(KERN_NOTICE "e100_config_loopback_mode:Invalid argument 'mode': %d\n",mode);
+    }
+    
+    spin_unlock_bh(&(bdp->config_lock));
+    return bc_changed;
+}
+
+
+/*
+ * Procedure:   e100_config_tcb_ext_enable
+ *
+ * Description: This routine will enable/disable the tcb extended mode
+ *              in the adapter's congif block, as indicated by the
+ *              tcb mode argument.
+ *
+ * Arguments:
+ *      bdp    - Ptr to this card's e100_bdconfig structure
+ *		enable   - tcb extended mode enable
+ *
+ * Returns: True -if the config block was changed
+ */
+
+unsigned char
+e100_config_tcb_ext_enable(bd_config_t *bdp, unsigned char enable)
+{
+    unsigned char	bc_changed = B_FALSE;
+
+    spin_lock_bh(&(bdp->config_lock));
+
+    if (enable) {
+        if( bdp->config[6] & CB_CFIG_EXT_TCB_DIS) {
+            
+            bdp->config[6] &= (~CB_CFIG_EXT_TCB_DIS);
+            E100_CONFIG(bdp, 6);
+            bc_changed =B_TRUE;
+       	} 
+
+    } else {
+        if( !(bdp->config[6] & CB_CFIG_EXT_TCB_DIS)) {
+            
+            bdp->config[6] |= CB_CFIG_EXT_TCB_DIS;
+            E100_CONFIG(bdp, 6);
+            bc_changed = B_TRUE;
+       	}
+    }
+    spin_unlock_bh(&(bdp->config_lock));
+    
+    return bc_changed;
+}
+
+/*
+ * Procedure:   e100_config_dynamic_tbd
+ *
+ * Description: This routine will enable/disable the dinamic tbd mode
+ *              in the adapter's congif block, as indicated by the
+ *             eboolean enable argument.
+ *
+ * Arguments:
+ *      bdp    - Ptr to this card's e100_bdconfig structure
+ *		enable   - True to enable
+ *
+ * Returns: True -if the config block was changed
+ */
+unsigned char
+e100_config_dynamic_tbd(bd_config_t *bdp,unsigned char enable)
+{
+    unsigned char	bc_changed = B_FALSE;
+    
+    spin_lock_bh(&(bdp->config_lock));
+    
+    if (enable) {
+        if( !(bdp->config[7] & CB_CFIG_DYNTBD_EN)) {
+            
+            bdp->config[7] |= CB_CFIG_DYNTBD_EN;
+            E100_CONFIG(bdp, 7);
+            bc_changed =B_TRUE;
+       	} 
+
+    } else {
+    	if( bdp->config[7] & CB_CFIG_DYNTBD_EN) {
+            
+            bdp->config[7] &= (~CB_CFIG_DYNTBD_EN);
+            E100_CONFIG(bdp, 7);
+            bc_changed =B_TRUE;
+       	} 
+    }
+    spin_unlock_bh(&(bdp->config_lock));
+
+    return bc_changed;
+}
+
+#endif /* IDIAG_PRO_SUPPORT */
