@@ -1,36 +1,75 @@
-/*****************************************************************************
- *****************************************************************************
- Copyright (c) 1999-2001, Intel Corporation 
+/**********************************************************************
+This software program is available to you under a choice of one of two 
+licenses. You may choose to be licensed under either the GNU General Public 
+License (GPL) Version 2, June 1991, available at 
+http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
+text of which follows:
 
- All rights reserved.
+Recipient has requested a license and Intel Corporation ("Intel") is willing
+to grant a license for the software entitled Linux Base Driver for the 
+Intel(R) PRO/100 Family of Adapters (e100) (the "Software") being provided 
+by Intel Corporation. The following definitions apply to this license:
 
- Redistribution and use in source and binary forms, with or without 
- modification, are permitted provided that the following conditions are met:
+"Licensed Patents" means patent claims licensable by Intel Corporation which 
+are necessarily infringed by the use of sale of the Software alone or when 
+combined with the operating system referred to below.
 
- 1. Redistributions of source code must retain the above copyright notice, 
- this list of conditions and the following disclaimer.
+"Recipient" means the party to whom Intel delivers this Software.
 
- 2. Redistributions in binary form must reproduce the above copyright notice,
- this list of conditions and the following disclaimer in the documentation 
- and/or other materials provided with the distribution.
+"Licensee" means Recipient and those third parties that receive a license to 
+any operating system available under the GNU Public License version 2.0 or 
+later.
 
- 3. Neither the name of Intel Corporation nor the names of its contributors 
- may be used to endorse or promote products derived from this software 
- without specific prior written permission.
+Copyright (c) 1999 - 2002 Intel Corporation.
+All rights reserved.
 
- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
- EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+The license is provided to Recipient and Recipient's Licensees under the 
+following terms.
+
+Redistribution and use in source and binary forms of the Software, with or 
+without modification, are permitted provided that the following conditions 
+are met:
+
+Redistributions of source code of the Software may retain the above 
+copyright notice, this list of conditions and the following disclaimer.
+
+Redistributions in binary form of the Software may reproduce the above 
+copyright notice, this list of conditions and the following disclaimer in 
+the documentation and/or materials provided with the distribution.
+
+Neither the name of Intel Corporation nor the names of its contributors 
+shall be used to endorse or promote products derived from this Software 
+without specific prior written permission.
+
+Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
+royalty-free patent license under Licensed Patents to make, use, sell, offer 
+to sell, import and otherwise transfer the Software, if any, in source code 
+and object code form. This license shall include changes to the Software 
+that are error corrections or other minor changes to the Software that do 
+not add functionality or features when the Software is incorporated in any 
+version of an operating system that has been distributed under the GNU 
+General Public License 2.0 or later. This patent license shall apply to the 
+combination of the Software and any operating system licensed under the GNU 
+Public License version 2.0 or later if, at the time Intel provides the 
+Software to Recipient, such addition of the Software to the then publicly 
+available versions of such operating systems available under the GNU Public 
+License version 2.0 or later (whether in gold, beta or alpha form) causes 
+such combination to be covered by the Licensed Patents. The patent license 
+shall not apply to any other combinations which include the Software. NO 
+hardware per se is licensed hereunder.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
+AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
+DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
+(INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
+ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
+AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
+TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
+OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+**********************************************************************/
 
- *****************************************************************************
-****************************************************************************/
 /*
 *                                                                     		*
 * This software is supplied under the terms of the license included   		*
@@ -80,55 +119,54 @@ __u16 GetEEpromSize(bd_config_t *adapter)
     __u16 x, size = 1;        // must be one to accumulate a product
 
     // if we've already stored this data, read from memory
-    if (adapter->EepromData.Size) {
-        size = adapter->EepromData.Size;
+    if (adapter->EEpromSize) {
+        return adapter->EEpromSize;
+    }
 
-    } else { // otherwise, read from the eeprom
-        // Set EEPROM semaphore.
-        if (adapter->rev_id >= D102_REV_ID) {
-            WAIT_FOR_CONDITION (adapter, EEPROM_SEMAPHORE_TIMEOUT,
-                                EepromSetSemaphore (adapter));
-        }
-
-        // enable the eeprom by setting EECS.
-        x = readw(&CSR_EEPROM_CONTROL_FIELD(adapter));
-        x &= ~(EEDI | EEDO | EESK);
-        x |= EECS;
+    // otherwise, read from the eeprom
+    // Set EEPROM semaphore.
+    if (adapter->rev_id >= D102_REV_ID) {
+        WAIT_FOR_CONDITION (adapter, EEPROM_SEMAPHORE_TIMEOUT,
+                            EepromSetSemaphore (adapter));
+    }
+    
+    // enable the eeprom by setting EECS.
+    x = readw(&CSR_EEPROM_CONTROL_FIELD(adapter));
+    x &= ~(EEDI | EEDO | EESK);
+    x |= EECS;
+    writew(x ,&CSR_EEPROM_CONTROL_FIELD(adapter));
+    
+    // write the read opcode
+    ShiftOutBits (adapter, EEPROM_READ_OPCODE, 3);
+    
+    // experiment to discover the size of the eeprom.  request register zero
+    // and wait for the eeprom to tell us it has accepted the entire address.
+    x = readw(&CSR_EEPROM_CONTROL_FIELD(adapter));
+    do {
+        size *= 2;          // each bit of address doubles eeprom size
+        x |= EEDO;          // set bit to detect "dummy zero"
+        x &= ~EEDI;         // address consists of all zeros
+        
         writew(x ,&CSR_EEPROM_CONTROL_FIELD(adapter));
-
-        // write the read opcode
-        ShiftOutBits (adapter, EEPROM_READ_OPCODE, 3);
-
-        // experiment to discover the size of the eeprom.  request register zero
-        // and wait for the eeprom to tell us it has accepted the entire address.
+        udelay (STALL_TIME);
+        RaiseClock (adapter, &x);
+        LowerClock (adapter, &x);
+        
+        // check for "dummy zero"
         x = readw(&CSR_EEPROM_CONTROL_FIELD(adapter));
-        do {
-            size *= 2;          // each bit of address doubles eeprom size
-            x |= EEDO;          // set bit to detect "dummy zero"
-            x &= ~EEDI;         // address consists of all zeros
-
-            writew(x ,&CSR_EEPROM_CONTROL_FIELD(adapter));
-            udelay (STALL_TIME);
-            RaiseClock (adapter, &x);
-            LowerClock (adapter, &x);
-
-            // check for "dummy zero"
-            x = readw(&CSR_EEPROM_CONTROL_FIELD(adapter));
-            if (size > EEPROM_MAX_WORD_SIZE) {
-                size = 0;
-                break;
-            }
-        } while (x & EEDO);
-
-        // read in the value requested
-        (void) ShiftInBits (adapter);
-        EEpromCleanup (adapter);
-
-        // Clear EEPROM Semaphore.
-        if (adapter->rev_id >= D102_REV_ID) {
-            EepromResetSemaphore (adapter);
+        if (size > EEPROM_MAX_WORD_SIZE) {
+            size = 0;
+            break;
         }
-
+    } while (x & EEDO);
+    
+    // read in the value requested
+    (void) ShiftInBits (adapter);
+    EEpromCleanup (adapter);
+    
+    // Clear EEPROM Semaphore.
+    if (adapter->rev_id >= D102_REV_ID) {
+        EepromResetSemaphore (adapter);
     }
 
     return size;
@@ -363,16 +401,16 @@ void EEpromCleanup(bd_config_t *adapter)
 // Returns: true/false  succeed/fail
 //----------------------------------------------------------------------------------------
 
-boolean_t ReadEEpromNodeAddress(bd_config_t *adapter, __u8 *buffer, unsigned int size)
+unsigned char ReadEEpromNodeAddress(bd_config_t *adapter, __u8 *buffer, unsigned int size)
 {
     __u16 value;
-    __u8  data [ETHERNET_ADDRESS_LENGTH];
+    __u8  data [ETH_ALEN];
     __u16 idx;
 
-    if (size < ETHERNET_ADDRESS_LENGTH)
+    if (size < ETH_ALEN)
         return FALSE;
 
-    for (idx = 0; idx < ETHERNET_ADDRESS_LENGTH / 2; idx++) {
+    for (idx = 0; idx < ETH_ALEN / 2; idx++) {
         value = ReadEEprom (adapter, (__u16) (EEPROM_NODE_ADDRESS_BYTE_0 + idx));
         data [idx*2]     = (__u8) value;
         data [idx*2 + 1] = (__u8) (value >> 8);
@@ -524,11 +562,6 @@ void WriteEEprom(bd_config_t *adapter, __u16 reg, __u16 data)
     if (adapter->rev_id >= D102_REV_ID) {
         EepromResetSemaphore (adapter);
     }
-
-    // update the eeprom buffer so subsequent reads will be accurate
-    if (adapter->EepromData.Initialized) {
-        adapter->EepromData.Data[reg] = data;
-    }
 }
 
 //----------------------------------------------------------------------------------------
